@@ -5,6 +5,8 @@ import socket
 from tqdm import tqdm
 from copy import deepcopy
 import strax
+from pprint import pprint
+
 try:
     import utilix
 except (RuntimeError, FileNotFoundError):
@@ -14,6 +16,7 @@ from straxen import uconfig
 
 export, __all__ = strax.exporter()
 
+
 @export
 class RunDB(strax.StorageFrontend):
     """
@@ -28,6 +31,7 @@ class RunDB(strax.StorageFrontend):
 
     def __init__(self,
                  minimum_run_number=7157,
+                 maximum_run_number=None,
                  runid_field='name',
                  local_only=False,
                  new_data_path=None,
@@ -39,29 +43,36 @@ class RunDB(strax.StorageFrontend):
                  mongo_database=None,
                  *args, **kwargs):
         """
-        :param mongo_url: URL to Mongo runs database (including auth)
-        :param local_only: Do not show data as available if it would have to be
-        downloaded from a remote location.
-        :param new_data_path: Path where new files are to be written.
-            Defaults to None: do not write new data
-            New files will be registered in the runs db!
-            TODO: register under hostname alias (e.g. 'dali')
+        :param minimum_run_number: Lowest number to consider
+        :param maximum_run_number: Highest number to consider. When None
+            (the default) consider all runs that are higher than the
+            minimum_run_number.
         :param runid_field: Rundb field to which strax's run_id concept
             corresponds. Can be either
             - 'name': values must be strings, for XENON1T
             - 'number': values must be ints, for XENONnT DAQ tests
-        :param reader_ini_name_is_mode: If True, will overwrite the 'mode'
-        field with 'reader.ini.name'.
+        :param local_only: Do not show data as available if it would
+            have to be downloaded from a remote location.
+        :param new_data_path: Path where new files are to be written.
+            Defaults to None: do not write new data
+            New files will be registered in the runs db!
+            TODO: register under hostname alias (e.g. 'dali')
+        :param reader_ini_name_is_mode: If True, will overwrite the
+            'mode' field with 'reader.ini.name'.
+        :param rucio_path: What is the base path where Rucio is mounted
+        :param mongo_url: URL to Mongo runs database (excl auth)
+        :param mongo_user: user to Mongo runs database
+        :param mongo_password: password to Mongo runs database
+        :param mongo_database: database name of Mongo runs database
 
         Other (kw)args are passed to StorageFrontend.__init__
-        #TODO
-        Add mongo_* to the docstring
         """
         super().__init__(*args, **kwargs)
         self.local_only = local_only
         self.new_data_path = new_data_path
         self.reader_ini_name_is_mode = reader_ini_name_is_mode
         self.minimum_run_number = minimum_run_number
+        self.maximum_run_number = maximum_run_number
         self.rucio_path = rucio_path
         if self.new_data_path is None:
             self.readonly = True
@@ -80,13 +91,13 @@ class RunDB(strax.StorageFrontend):
 
         # setup mongo kwargs...
         # utilix.rundb.pymongo_collection will take the following variables as kwargs
-        # url: mongo url, including auth
-        # user: the user
-        # password: the password for the above user
-        # database: the mongo database name
+        #     url: mongo url, including auth
+        #     user: the user
+        #     password: the password for the above user
+        #     database: the mongo database name
         # finally, it takes the collection name as an arg (not a kwarg).
         # if no collection arg is passed, it defaults to the runsDB collection
-        # See https://github.com/XENONnT/utilix/blob/master/utilix/rundb.py for more details
+        # See github.com/XENONnT/utilix/blob/master/utilix/rundb.py for more details
         mongo_kwargs = {'url': mongo_url,
                         'user': mongo_user,
                         'password': mongo_password,
@@ -113,7 +124,9 @@ class RunDB(strax.StorageFrontend):
             self.backends.append(strax.rucio(self.rucio_path))
             # When querying for rucio, add that it should be dali-userdisk
             self.available_query.append({'host': 'rucio-catalogue',
-                                         'location': 'UC_DALI_USERDISK'})
+                                         'location': 'UC_DALI_USERDISK',
+                                         'status': 'transferred',
+                                         })
 
     def _data_query(self, key):
         """Return MongoDB query for data field matching key"""
@@ -121,8 +134,17 @@ class RunDB(strax.StorageFrontend):
             'data': {
                 '$elemMatch': {
                     'type': key.data_type,
-                    'meta.lineage': key.lineage,
-                    '$or': self.available_query}}}
+                    # TODO remove the meta.lineage since this doc
+                    #  entry is deprecated.
+                    '$and': [{'$or': [
+                        {'meta.lineage': key.lineage},
+                        {'did':
+                             {'$regex':
+                                  f'/*{key.data_type}-{key.lineage_hash}'
+                              },
+                         },
+                    ]},
+                        {'$or': self.available_query}]}}}
 
     def _find(self, key: strax.DataKey,
               write, allow_incomplete, fuzzy_for, fuzzy_for_options):
@@ -138,19 +160,26 @@ class RunDB(strax.StorageFrontend):
         # Check that we are in rucio backend
         if self.rucio_path is not None:
             rucio_key = self.key_to_rucio_did(key)
+            rucio_available_query = self.available_query[-1]
             dq = {
                 'data': {
                     '$elemMatch': {
-                        # TODO can we query smart on the lineage_hash?
                         'type': key.data_type,
                         'did': rucio_key,
-                        'protocol': 'rucio'}}}
-            doc = self.collection.find_one({**run_query, **dq},
+                        **rucio_available_query,
+                    },
+                }}
+            doc = self.collection.find_one({**run_query,
+                                            **dq,
+                                            },
                                            projection=dq)
             if doc is not None:
                 datum = doc['data'][0]
-                assert datum.get('did', '') == rucio_key, f'Expected {rucio_key} got data on {datum["location"]}'
-                backend_name, backend_key = datum['protocol'], f'{key.run_id}-{key.data_type}-{key.lineage_hash}'
+                error_message = f'Expected {rucio_key} got data on {datum["location"]}'
+                assert datum.get('did', '') == rucio_key, error_message
+                backend_name, backend_key = (
+                    datum['protocol'],
+                    f'{key.run_id}-{key.data_type}-{key.lineage_hash}')
                 return backend_name, backend_key
 
         dq = self._data_query(key)
@@ -166,7 +195,8 @@ class RunDB(strax.StorageFrontend):
             if self.new_data_path is not None:
                 doc = self.collection.find_one(run_query, projection={'_id'})
                 if not doc:
-                    raise ValueError(f"Attempt to register new data for non-existing run {key.run_id}")   # noqa
+                    raise ValueError(f"Attempt to register new data for"
+                                     f" non-existing run {key.run_id}")
                 self.collection.find_one_and_update(
                     {'_id': doc['_id']},
                     {'$push': {'data': {
@@ -180,16 +210,19 @@ class RunDB(strax.StorageFrontend):
 
             return (strax.FileSytemBackend.__name__,
                     output_path)
-
         datum = doc['data'][0]
 
+        if datum['host'] == 'rucio-catalogue':
+            # TODO this is due to a bad query in _data_query. We aren't rucio.
+            raise strax.DataNotAvailable
+
         if write and not self._can_overwrite(key):
             raise strax.DataExistsError(at=datum['location'])
 
         return datum['protocol'], datum['location']
 
     def find_several(self, keys: typing.List[strax.DataKey], **kwargs):
-        if kwargs['fuzzy_for'] or kwargs['fuzzy_for_options']:
+        if kwargs.get('fuzzy_for', False) or kwargs.get('fuzzy_for_options', False):
             raise NotImplementedError("Can't do fuzzy with RunDB yet.")
         if not len(keys):
             return []
@@ -197,7 +230,7 @@ class RunDB(strax.StorageFrontend):
             raise ValueError("find_several keys must have same lineage")
         if not len(set([k.data_type for k in keys])) == 1:
             raise ValueError("find_several keys must have same data type")
-        keys = list(keys)   # Context used to pass a set
+        keys = list(keys)  # Context used to pass a set
 
         if self.runid_field == 'name':
             run_query = {'name': {'$in': [key.run_id for key in keys]}}
@@ -222,8 +255,10 @@ class RunDB(strax.StorageFrontend):
                 dk = doc['name']
             else:
                 dk = f'{doc["number"]:06}'
-
-            results_dict[dk] = datum['protocol'], datum['location']
+            try:
+                results_dict[dk] = datum['protocol'], datum['location']
+            except KeyError as e:
+                raise KeyError(f'Queries failed\n{run_query}\n{dq}\n{doc}') from e
         return [results_dict.get(k.run_id, False)
                 for k in keys]
 
@@ -235,23 +270,20 @@ class RunDB(strax.StorageFrontend):
             raise NotImplementedError("Can't allow_incomplete with RunDB yet")
 
         q = self._data_query(key)
-        if self.minimum_run_number:
-            q['number'] = {'$gt': self.minimum_run_number}
+        q.update(self.number_query())
+
         cursor = self.collection.find(
             q,
             projection=[self.runid_field])
         return [x[self.runid_field] for x in cursor]
 
     def _scan_runs(self, store_fields):
-        if self.minimum_run_number:
-            query = {'number': {'$gt': self.minimum_run_number}}
-        else:
-            query = {}
+        query = self.number_query()
         projection = strax.to_str_tuple(list(store_fields))
         # Replace fields by their subfields if requested only take the most
         # "specific" projection
         projection = [f1 for f1 in projection
-                      if not any([f2.startswith(f1+".") for f2 in projection])]
+                      if not any([f2.startswith(f1 + ".") for f2 in projection])]
         cursor = self.collection.find(
             filter=query,
             projection=projection)
@@ -282,6 +314,16 @@ class RunDB(strax.StorageFrontend):
             doc['mode'] = doc.get('reader', {}).get('ini', {}).get('name', '')
         return doc
 
+    def number_query(self):
+        q_number = {}
+        if self.minimum_run_number:
+            q_number = {'$gt': self.minimum_run_number}
+        if self.maximum_run_number is not None:
+            q_number.update({'$lt': self.maximum_run_number})
+        if q_number:
+            return {'number': q_number}
+        return {}
+
     @staticmethod
     def key_to_rucio_did(key: strax.DataKey):
         """Convert a strax.datakey to a rucio did field in rundoc"""
